/*! \file
 *
 * SOC Robotics, Inc
 *
 * File - 7SegmentDisplay.c
 *
 * Compiler - ICCAVR 7.16a
 *
 * Description  - This file sets the device to act as a TWI Slave
 * It will process co
 *
 * 0xF9DFE6 => External clock
 *
 * $Revision: 1.0.1 $
 * $Date: June 2nd, 2008 $
 *
 *   For 20MHz external operation:  
 *     Fuse settings should be:  F9 D9 E7
 *
 * Copyright 2008, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC ROBOTICS, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com/
 */

#include <stdio.h>
 
 
#include <iom168v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
#include "Board.h"
#include "Timer.h" 
#include "Serial.h"
#include "TWI_slave.h"
#include "LED_Display.h"

unsigned char CheckEEPROMWritten(void);

void Init_EEPROM(void);

void  LoadEEPROMSettings(void);

// Function to initiate an EEPROM write
// writes the specified data byte to the specified location
// this will fail if the EEPROM is not ready!
void RTEEPROMwrite(int location, unsigned char databyte);

// EEPROM writes take about 8ms - so poll until done
// Do not use ICC V7 function EEPROMwrite() - this routine turns interrupts off!
void EEPROMWrite(int location, unsigned char databyte);

// Write a string of characters to EEPROM at specified start location
void WriteEEPROM(int location, unsigned char *strg);
// function to test if the EEPROM is ready for a read or write operation
// returns non zero if ready, zero if not ready 
unsigned char RTEEPROMReady(void);

unsigned char RTEEPROMread(int location);


void TWIPeripheral(void);
unsigned char TimeToUpdate(void);

void watchdog_init(void);


float positionScale = DEFAULT_POSITION_SCALE;
int decimalPlace = DEFAULT_DECIMAL_PLACE;

void main(void)
{
	int ii;
	CLI();//disable interrupts
    
    
    
	Buttons_Init();
    LED_Init(); 
    uart0_init();//incase we get an error in the TWI
   // LoadEEPROMSettings();
   // DDRD |= (1<<0); //hearth beat output
    
	Init_Timer();
	Init_Timer0();
    
	TWI_Slave_Initialise( TWI_ADDRESS);

 //   watchdog_init();
  
	SEI(); // Enable interrupts

	//wait to show the display is working
	mseccountdown1 = 1000;
	while(mseccountdown1 != 0)
    {
    	;//WDR ();
    }
	uart0_puts ("\nStart Peripheral"); 
    
	LED_Clear();  
	
	TWIPeripheral();
}

/*!
 * \brief This function will check to see if it has received new data from
 * the master.  If it has it will process the data.  It will also update
 * the values of the boards buttons into the TWI output buffer
 */
void TWIPeripheral(void)
{
	unsigned char i;

	unsigned char temp[TWI_RX_BUFFER_SIZE];
	 
	//This void pointer is used to cast the transmitted data to the correct type
	void *value;

	while(1)
	{
    	WDR ();
        
		//if we have received new data, process it
		if(TWI_Get_Data_From_Transceiver( temp, TWI_RX_BUFFER_SIZE ))
		{
			value = &temp[1];//my data if any will start at the next byte
				
			switch(temp[0])//what command was sent?
			{
				case COMMAND_7SEGMENT_WRITE_INT:
				case COMMAND_7SEGMENT_WRITE_INT_B:
					//Before reading the data, I cast it to the correct type
					Display_Int(*(int *)value, 0, DEC);
					break;
					
				case COMMAND_7SEGMENT_WRITE_HEX_INT:
				case COMMAND_7SEGMENT_WRITE_HEX_INT_B:
					Display_Int(*(int *)value, 0, HEX);
					break;
						
				case COMMAND_7SEGMENT_WRITE_BIN_INT:
				case COMMAND_7SEGMENT_WRITE_BIN_INT_B:
					Display_Int(*(int *)value, 0, BIN);
					break;
						
				case COMMAND_7SEGMENT_WRITE_FLOAT:
				case COMMAND_7SEGMENT_WRITE_FLOAT_B:
					Display_Float(*(float *)value,0, decimalPlace);
					break;
						
				case COMMAND_7SEGMENT_WRITE_POSITION:
				case COMMAND_7SEGMENT_WRITE_POSITION_B:
					Display_Float((*(long *)value)*positionScale,0, decimalPlace);
					break;
						
				case COMMAND_7SEGMENT_SET_POSITION_SCALE:
				case COMMAND_7SEGMENT_SET_POSITION_SCALE_B:
					positionScale = *(float *)value;
					break;
						
				case COMMAND_7SEGMENT_SET_DECIMAL_PLACE:
				case COMMAND_7SEGMENT_SET_DECIMAL_PLACE_B:
					decimalPlace = *(int *)value;
					break;
				//If the command is not recognised, simply display it		
				default:
					Display_Int(temp[0], 0, HEX);
			}
		}
			  
		//Is it time we update our button values?
		if(TimeToUpdate())
		{
			updateButtons();
			temp[0] = currentButtonValue;
			//Add button values
			TWI_Transceiver_Update_Data( temp, 1 );
		}
			 
	}
}

/*!
 * \brief This function checks a timer to see if it has counted down
 * If it has it will restart the timer and return true
 */
unsigned char TimeToUpdate(void)
{
	if(mseccountdown1 == 0)
	{
		mseccountdown1 = 10;
		return TRUE;
	}
	return FALSE;				 		 
}


//Watchdog initialize
// prescale: 2K
void watchdog_init(void)
{
 WDR (); //this prevents a timeout on enabling
 
 
 //WDTCSR = (1<<WDIE) | (1<<WDCE) | (1<<WDE) | (7<<0);/* 30-Oct-2006 Umesh*/  
 
 WDTCSR |= (1<<WDCE);
 WDTCSR = 0x4F; //WATCHDOG ENABLED - dont forget to issue WDRs
}

#pragma interrupt_handler wdt_isr:iv_WDT
void wdt_isr(void)
{
	uart0_puts ("\nWatchDog Timer");
 //watchdog timeout
}

unsigned char CheckEEPROMWritten()
{
 	char signature[19];
	unsigned char i;
	
	signature[18] = 0;
	for(i=0; i<18; i++)
    {
    	signature[i] = RTEEPROMread(i);//EEPROMread(i);
    }
    uart0_puts("\nRead PROM\n");
    uart0_puts(&signature[0]);
    uart0_puts("\n--------\n");
	if(!strcmp(signature,"SOC Robotics V0.87")) return 1;
	else return 0;
}

void Init_EEPROM()
{
	WriteEEPROM(0,"SOC Robotics V0.87");
	//EEPROMWrite(19,'d');   // Write calibration byte
	//EEPROMWrite(20,'3');      // Set baud rate to 38400
}

void  LoadEEPROMSettings(void)
{
	//  See if EEPROM setup values exist - if not write them 
	//  otherwise load settings
	if(CheckEEPROMWritten()==0)
    {
    	
        uart0_puts ("\nWrote Out Values\n");
        // Write default values to EEPROM
		Init_EEPROM();
	}
	else
    {
    	uart0_puts ("\nReading Values\n");
	   // OSCCAL = EEPROMread(19);
		//SetNewBaudRate(EEPROMread(20));
		//puts("use eeprom settings\n\r");
    }
    
    while(!RTEEPROMReady());
}

// Function to initiate an EEPROM write
// writes the specified data byte to the specified location
// this will fail if the EEPROM is not ready!
void RTEEPROMwrite(int location, unsigned char databyte)
{
	unsigned char savedSREG;
	EEAR = location;								// set address
	EEDR = databyte;								// set data	
	savedSREG = SREG;								// keep setting so it can be restored
	CLI();													// disable interrupts
	EECR |= BIT(EEMWE);							// set "write enable" bit
	EECR |= BIT(EEWE);							// set "write" bit
	SREG = savedSREG;								// restore SREG
	EEAR = 0;
}

// EEPROM writes take about 8ms - so poll until done
void EEPROMWrite(int location, unsigned char databyte)
{
 	 while(!RTEEPROMReady())  ;
 	 RTEEPROMwrite(location,databyte);
 	 while(!RTEEPROMReady())  ;
}

unsigned char RTEEPROMread(int location)
{
	while(!RTEEPROMReady())  ;
	EEAR = location;								// set address
	EECR |= BIT(EERE);							// set "read enable" bit
	EEAR = 0;
	return (EEDR);
}

// Write a string of characters to EEPROM at specified start location
void WriteEEPROM(int location, unsigned char *strg)
{
	int i;
	 
	i = 0;
	while(strg[i] != 0x00)
	{
		EEPROMWrite(location+i,strg[i]);
		i++;
	}
}

// function to test if the EEPROM is ready for a read or write operation
// returns non zero if ready, zero if not ready 
unsigned char RTEEPROMReady(void)
{
	return !(EECR & 0x02);
}

